@@ -1,5 +1,5 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8; coding: utf-8 -*-
- *  gtksourcelanguage.c
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8; coding: utf-8 -*- */
+/*  gtksourcelanguage.c
  *
  *  Copyright (C) 2003 - Paolo Maggi <paolo.maggi@polito.it>
  *
@@ -39,6 +39,18 @@
 #include "gtksourcelanguage.h"
 #include "gtksourceview-marshal.h"
 
+/**
+ * SECTION:language
+ * @Short_description: Object representing a syntax highlighted language
+ * @Title: GtkSourceLanguage
+ * @See_also: #GtkSourceLanguageManager
+ *
+ * #GtkSourceLanguage encapsulates syntax and highlighting styles for a
+ * particular language. Use #GtkSourceLanguageManager to obtain a
+ * #GtkSourceLanguage instance, and gtk_source_buffer_set_language() to apply it
+ * to a #GtkSourceBuffer.
+ */
+
 #define DEFAULT_SECTION _("Others")
 
 /* Properties */
@@ -52,8 +64,9 @@ enum {
 
 G_DEFINE_TYPE (GtkSourceLanguage, gtk_source_language, G_TYPE_OBJECT)
 
-static GtkSourceLanguage *process_language_node 		(xmlTextReaderPtr 		 reader,
-								 const gchar 			*filename);
+static GtkSourceLanguage *process_language_node (xmlTextReaderPtr	 reader,
+						 const gchar		*filename);
+static gboolean		  force_styles		(GtkSourceLanguage	*language);
 
 GtkSourceLanguage *
 _gtk_source_language_new_from_file (const gchar              *filename,
@@ -147,7 +160,7 @@ gtk_source_language_get_property (GObject    *object,
 			break;
 
 		case PROP_SECTION:
-			g_value_set_string (value, language->priv->name);
+			g_value_set_string (value, language->priv->section);
 			break;
 
 		case PROP_HIDDEN:
@@ -217,7 +230,7 @@ gtk_source_language_class_init (GtkSourceLanguageClass *klass)
 							      G_PARAM_READABLE));
 
 	g_object_class_install_property (object_class,
-					 PROP_ID,
+					 PROP_NAME,
 					 g_param_spec_string ("name",
 						 	      _("Language name"),
 							      _("Language name"),
@@ -225,7 +238,7 @@ gtk_source_language_class_init (GtkSourceLanguageClass *klass)
 							      G_PARAM_READABLE));
 
 	g_object_class_install_property (object_class,
-					 PROP_ID,
+					 PROP_SECTION,
 					 g_param_spec_string ("section",
 						 	      _("Language section"),
 							      _("Language section"),
@@ -274,11 +287,10 @@ static void
 process_properties (xmlTextReaderPtr   reader,
 		    GtkSourceLanguage *language)
 {
-	gint ret;
 	xmlNodePtr child;
 	xmlNodePtr node = NULL;
 
-	while (node == NULL && (ret = xmlTextReaderRead (reader)) == 1)
+	while (node == NULL && xmlTextReaderRead (reader) == 1)
 	{
 		xmlChar *name;
 
@@ -555,7 +567,6 @@ gtk_source_language_get_metadata (GtkSourceLanguage *language,
 
 /**
  * gtk_source_language_get_mime_types:
- *
  * @language: a #GtkSourceLanguage.
  *
  * Returns the mime types associated to this language. This is just
@@ -583,7 +594,6 @@ gtk_source_language_get_mime_types (GtkSourceLanguage *language)
 
 /**
  * gtk_source_language_get_globs:
- *
  * @language: a #GtkSourceLanguage.
  *
  * Returns the globs associated to this language. This is just
@@ -625,6 +635,15 @@ _gtk_source_language_get_language_manager (GtkSourceLanguage *language)
 
 /* Highlighting engine creation ------------------------------------------ */
 
+static void
+copy_style_info (const char         *style_id,
+		 GtkSourceStyleInfo *info,
+		 GHashTable         *dest)
+{
+	g_hash_table_insert (dest, g_strdup (style_id),
+			     _gtk_source_style_info_copy (info));
+}
+
 void
 _gtk_source_language_define_language_styles (GtkSourceLanguage *lang)
 {
@@ -643,6 +662,8 @@ _gtk_source_language_define_language_styles (GtkSourceLanguage *lang)
 		{NULL, NULL}};
 
 	gint i = 0;
+	GtkSourceLanguageManager *lm;
+	GtkSourceLanguage *def_lang;
 
 	while (alias[i][0] != NULL)
 	{
@@ -656,13 +677,26 @@ _gtk_source_language_define_language_styles (GtkSourceLanguage *lang)
 
 		++i;
 	}
+
+	/* We translate String to def:string, but def:string is mapped-to
+	 * def:constant in def.lang, so we got to take style mappings from def.lang */
+
+	lm = _gtk_source_language_get_language_manager (lang);
+	def_lang = gtk_source_language_manager_get_language (lm, "def");
+
+	if (def_lang != NULL)
+	{
+		force_styles (def_lang);
+		g_hash_table_foreach (def_lang->priv->styles,
+				      (GHFunc) copy_style_info,
+				      lang->priv->styles);
+	}
 }
 
-GtkSourceEngine *
-_gtk_source_language_create_engine (GtkSourceLanguage *language)
+/* returns new reference, which _must_ be unref'ed */
+static GtkSourceContextData *
+gtk_source_language_parse_file (GtkSourceLanguage *language)
 {
-	GtkSourceContextEngine *ce = NULL;
-
 	if (language->priv->ctx_data == NULL)
 	{
 		gboolean success = FALSE;
@@ -686,6 +720,9 @@ _gtk_source_language_create_engine (GtkSourceLanguage *language)
 				case GTK_SOURCE_LANGUAGE_VERSION_2_0:
 					success = _gtk_source_language_file_parse_version2 (language, ctx_data);
 					break;
+
+				default:
+					g_assert_not_reached ();
 			}
 
 			if (!success)
@@ -699,10 +736,21 @@ _gtk_source_language_create_engine (GtkSourceLanguage *language)
 		_gtk_source_context_data_ref (language->priv->ctx_data);
 	}
 
-	if (language->priv->ctx_data)
+	return language->priv->ctx_data;
+}
+
+GtkSourceEngine *
+_gtk_source_language_create_engine (GtkSourceLanguage *language)
+{
+	GtkSourceContextEngine *ce = NULL;
+	GtkSourceContextData *ctx_data;
+
+	ctx_data = gtk_source_language_parse_file (language);
+
+	if (ctx_data != NULL)
 	{
-		ce = _gtk_source_context_engine_new (language->priv->ctx_data);
-		_gtk_source_context_data_unref (language->priv->ctx_data);
+		ce = _gtk_source_context_engine_new (ctx_data);
+		_gtk_source_context_data_unref (ctx_data);
 	}
 
 	return ce ? GTK_SOURCE_ENGINE (ce) : NULL;
@@ -761,19 +809,20 @@ get_style_ids (GtkSourceLanguage *language)
 static gboolean
 force_styles (GtkSourceLanguage *language)
 {
-	GtkSourceEngine *ce;
-
-	/* To be sure to have the list of styles we need to create a
-	 * GtkSourceContextEngine.
-	 * In the future we can improve this avoiding to create a context
-	 * engine.
+	/* To be sure to have the list of styles we need to parse lang file
+	 * as if we were to create an engine. In the future we can improve
+	 * this by parsing styles only.
 	 */
-	if (language->priv->ctx_data == NULL)
+	if (!language->priv->styles_loaded && language->priv->ctx_data == NULL)
 	{
-		ce = _gtk_source_language_create_engine (language);
-		if (ce == NULL)
+		GtkSourceContextData *ctx_data;
+
+		ctx_data = gtk_source_language_parse_file (language);
+		if (ctx_data == NULL)
 			return FALSE;
-		g_object_unref (ce);
+
+		language->priv->styles_loaded = TRUE;
+		_gtk_source_context_data_unref (ctx_data);
 	}
 
 	return TRUE;
@@ -781,7 +830,6 @@ force_styles (GtkSourceLanguage *language)
 
 /**
  * gtk_source_language_get_style_ids:
- *
  * @language: a #GtkSourceLanguage
  *
  * Returns the ids of the styles defined by this @language.
@@ -819,7 +867,6 @@ get_style_info (GtkSourceLanguage *language, const char *style_id)
 
 /**
  * gtk_source_language_get_style_name:
- *
  * @language: a #GtkSourceLanguage
  * @style_id: a style ID
  *
@@ -830,9 +877,9 @@ get_style_info (GtkSourceLanguage *language, const char *style_id)
  * by this @language. The returned string is owned by the @language and must
  * not be modified.
  */
-const char *
+const gchar *
 gtk_source_language_get_style_name (GtkSourceLanguage *language,
-				    const char        *style_id)
+				    const gchar       *style_id)
 {
 	GtkSourceStyleInfo *info;
 
@@ -860,6 +907,13 @@ _gtk_source_style_info_new (const gchar *name, const gchar *map_to)
 	return info;
 }
 
+GtkSourceStyleInfo *
+_gtk_source_style_info_copy (GtkSourceStyleInfo *info)
+{
+	g_return_val_if_fail (info != NULL, NULL);
+	return _gtk_source_style_info_new (info->name, info->map_to);
+}
+
 void
 _gtk_source_style_info_free (GtkSourceStyleInfo *info)
 {
